% Load the PCA data, and store the value of the dataset identifier to variable "pid"
% Note that I used the variable "pca", you could easily use a different
% name becasue it really doesn't matter what you call it, it's just a variable

variable pid = load_data("pca.pha");
% do a  >list_data;  to make sure background etc files are included

% Put (conservative) 0.5%, systematic errors since PCA has very good statistics

set_sys_err_frac(pid,Double_Type[129]+0.005);

% Group into bins with minimum S/N of 4.5

group(pid;min_sn=4.5,bounds=3.,unit="kev");

% notice only bins with energy between 3-22 keV

notice_values(pid,3.,22.;unit="kev");

%now plot data after grouping
% set log axes 
xlog;ylog;

%set point size bigger
point_size(1.5);

% set line width for data and error bars bigger so it shows up darker
% (you can also do this for residuals, model etc...use the usual syntax
% of typing >set_plot_widths;  to get full list of options

set_plot_widths(;d_width=5,de_width=5);


% plot data (counts/s/en) with and without background (overplotted).  You can use
%>pginfo to find out all the various toggles and values possible
 
plot_data(pid;dsym=4,dcol=8,decol=7,bkg=1); 
plot_data(pid;dsym=4,dcol=4,decol=12,oplt=1);

% now the same for HEXTE

variable hid = load_data("hxt.pha");
group(hid;min_sn=4.5,bounds=20.,unit="kev");
notice_values(hid,20.,200.;unit="kev");

% play around a bit, plot with and without background.  Very different
%  than PCA and illuminating for backgrounds
% NULL allows "autoscale"

plot_data(hid;xrange={20,200},yrange={0.005,2.2},  dsym=4,dcol=8,decol=7, bkg=1);
plot_data(hid;dsym=4,dcol=4,decol=5,oplt=1);

%now plot PCA and HEXTE together.  Now you need to pass vectors to the
% plotting routines, delineated by {}'s.   But NULL won't work well so
% give explicit values for x and y axes

plot_data({pid,hid};dsym=[-4,4],dcol=[4,8], decol=[5,7],xrange={3,200},yrange={0.005,30});


% but it's important to note that HEXTE and PCA don't agree on
%  normalization, we know this because they overlap in energy.   For
% example, with Suzaku CCD's stopping (effectively) at 8 keV (at 10
% keV no matter what), and the hard X-ray detector not starting until
% 15 keV (effectively; 12 keV no matter what), it might be more
% difficult to tell what the cross normalization is.  The radio, being
% far, far away, has no overlap.  Therefore, a priori, we don't know
% whether it agrees better with PCA or with HEXTE. 
% If we were *really* good, we'd fit models with each as the "true
% norm", and see how that effects our answer.

% plot unfolded spectra to show normalization

%The plot_unfold function will make use of the power qualifier or
%structure field. Choosing power=1 means being proportional to
%photons/area/sec/(unit x), and then power=0 means dividing by an-
%other power of x, power=2–4 means multiplying by additional powers of
%x. This allows for plotting things like Fλ, λFλ, νFν, etc.

fancy_plot_unit("kev", "ergs");
plot_unfold({pid,hid};dsym={-4,4},dcol={4,8},decol={5,7},xrange={NULL,NULL},yrange={NULL,NULL},power=3);




% Now you could pause to fit these spectra (maybe do quick PL fit)
%  but main point is that eventually we want to load the radio and IR
%  data, from an ascii file in format Freq (Hz), Flux (mJy)  Flux error (mJy)

variable rid_struct = read_radio("53082_radio-OIR.dat",1000);
variable rid = load_radio(rid_struct);


% Look at the code for these functions.  Basically,
% what we are doing is, if *any* dataset is input to ISIS *without* an
% ARF & RMF, it assumes RMF = diagonal matrix, with values = 1, and
% ARF = 1 cm^2, and "exposure" = 1 sec.  Thus, we need to convert the
% radio flux into counts/1 cm^2/1 sec.  Note also that the *only real
% reason* that we do that is that all the XSPEC models assume
% counts/bin/sec.  ISIS doesn't *really* care that the ARF is 1 cm^2,
% RMF is diagonal with values of 1, and exposure is 1 sec.  That's how
% it interfaces with the XSPEC models to give them a value for the
% fit.   So, you can write models that have no regards for "units" per
% se.  ***HOWEVER** all the XSPEC models calculate counts/cm^2/sec 
% per energy bin.  So, since you want to fit XSPEC models to both
% X-ray data & radio data,  we have to convert the radio data to the
% units of the XSPEC model: counts/bin.  If we were *just* fitting the
% radio, we could write an *independent* set of models that were 
%normalized in mJy, say.


% But if we want to switch to more "radio astronomy" units, we should
%  rather consider mJy units or even Hz and mjy.  Luckily Fancy Plots
%  makes this very easy

fancy_plot_unit("hz","mjy");

% You can also cut down on typing by using a globally defined structure to
%  input all your plotting variables to save typing effort

popt.dsym={5,-4,4};
popt.dcol={12,4,8}; 
popt.decol={3,5,7}; 
popt.bkg={0,0,0}; 
popt.mcol={0,0,0};
popt.xrange={3.e8,1.e20};
popt.yrange={0.001,100.};
popt.power=2.;

plot_unfold({rid,pid,hid},popt);


% now if we want to fit the data we need to set up grids, where each
%  grid has its own normalization.  So we group radio/IR with pca on one
%  grid, and hexte on its own.  We use a script defined in the
%  .isisrc* files

usr_grid([rid,pid],-9,3,0.001,1);

% parameters are log10 min, log10max (in kev), grid spacing in log, cache model
%  on grid

usr_grid([hid],0,3,0.001);


