%  this file is where you put everything about the model you want to
%   load.  

% allow using variables not declared first
_auto_declare = 1;


%  first define a couple useful finger-saving aliases

alias("eval_counts","ec");
alias("list_par","lp");
alias("newpar", "np");
alias("set_par", "sp");
alias("fit_counts", "fit");

% and let's set the model width high so we can see it well, and get
%  ready for plotting residuals

set_plot_widths(;m_width=5, r_width=5, re_width=5);

% Let's first exclude the radio data, it's still there but we won't
%  use it in the model evaluation for now

exclude(rid);

% Let's use a very simple phenomenological model to describe the data,
%  a broken power law, with some absorption along the line of sight

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*bknpower(1)");

% look at parameters

lp;  % aliased from list_par above

%the constants are the normalization between the grids.  We want to
% fix the PCA to 1 and then let const(2) for hexte vary against it.

set_par(1,1,1);  %now we'll use alias sp

% limit the other constant to values around 1 since we know the
%  normalization between the two instruments is pretty good

sp(7,1.,0,.9,1.1);


% freeze N_H to the "known" value from the literature

sp(2,0.6,1);

% plot data again

fancy_plot_unit("keV","mjy");
plot_unfold({pid,hid};dsym=[-4,4],dcol=[4,8],decol=[5,7],xrng={2.,300.},yrng={2.e-3,.2},power=2);

%  invalid/failed complaints only mean some parts are out of the
%   bounds of the fits, not a problem as long as you're plotting the
%   range you want

% do a single evaluation and then spend a bit of time changing pars by hand
%  and see how far you can get using np changes

ec;
renorm_counts;

% I can get it to about chi2 of 20 or so...

% let's look at residuals:

set_plot_widths(;r_width=5,re_width=5, m_width=5);

plot_unfold({pid,hid};dsym=[-4,4],dcol=[4,8],decol=[5,7],rcol=[4,8],recol=[5,7],xrange={2,200},yrange={0.002,.2,-10,10},res=1);

% pretty bad.  Clearly we're missing a line and a cutoff so let's
% change the model to something more sophisticated and this time do an actual fit routine to optimize

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*highecut(1)*(bknpower(1)+diskbb(1)+ gaussian(1))");

% Exponential cutoffs start around 20 keV, and have typical rollover
% energies of around 150 keV, so good guess to start.

sp(3,20,0,6,200);  % Where the cutoff starts - wouldn't let it go below Fe,
                                     % or above the upper cutoff of HEXTE

sp(4,150,0,30,500);  % 30-500 is a generous range to allow

% Typical value for line normalizations, energy, and width

sp(11,5.e-4,0,0,0.1);
sp(12,6.4,0,6,7);  % Don't want the line energy running off too far
sp(13,0.3,0,0,1);  % nor getting too wide

% Now let's do a real fit:

fit_counts;

% if this returns -1 then it didn't converge and you can rerun.  You
%  can also type.  Mine converged but got 1.3 so not amazing


%subplex will give a more thorough (and slower) fitting routine, but
%  we won't use that, this is good enough for now.  You can run it for
%  awhile and stop it with ^c even if it doesn't  finish and then
%  switch back to lmdir, refit and get something better.  I got down
%  to 0.92

% now replot with residuals (have to redefine popt since went from 3 data
%  sets to 2

popt.dsym={-4,4};
popt.dcol={4,8};
popt.decol={5,7};
popt.rcol={4,8};
popt.recol={5,7};
popt.mcol={2,2};
popt.xrange={NULL,NULL};
popt.yrange={NULL,NULL,-10,10};
popt.res=1;

plot_unfold({pid,hid},popt);

% obviously missing some components.  Let's try a more sophisticated
% model:

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*highecut(1)*(bknpower(1)+diskbb(1)
+ gaussian(1))");

fit_counts;


% not bad, same as before, let's renotice the radio, fix constants, evaluate the model, see that it sucks, and plot it to verify that it sucks.  

include(rid);
lp;
sp(15,1,1);
tie(15,1);
ec;

%replot

plot_unfold({rid,pid,hid};dsym={5,-4,4},dcol={12,4,8},decol={3,5,7},rcol={12,4,8},recol={3,5,7},mcol={2,2,2},xrange={3.e-9,250},yrange={1.e-3,50,-2.e5,2.e5},power=2,rsym={5,-4,4},res=1);


% bkpln is way off in radio...assuming that you want to fit it with
%  some similar process

%Let's keep these plot options in our popt. so we don't have to keep
%typing/copying it:

popt.dsym={5,-4,4};
popt.dcol={12,4,8}; 
popt.decol={3,5,7};
popt.rcol={12,4,8};
popt.recol={3,5,7};
popt.mcol={2,2,2}; 
popt.xrange={3.e-9,250};
popt.yrange={1.e-3,50,-5,5};
popt.power=2;
popt.rsym={5,-4,4};
popt.res=1;

%  Now let's start thinking about real physics. So right now you have this
%  magic broken cutoff PL that doesn't really mean anything, and then you
%  have a couple of "real" components, like a blackbody and a line
%  that are representing an accretion disk and fluorescent iron.
%  Let's start moving towards what we would expect a physical model to
%  look like.  

% This data set we're looking at is from GX339-4, it's a Galactic
%  X-ray binary in it's "hard state", the state associated with
%  compact jets.  We already heard that the synchrotron from jets can
%  extend over orders of magnitude, but why would we expect a flat
%  spectrum in the radio?  

%  See slides on jets, lead into:  let's approximate the jet
%  contribution as the broken powerlaw, so assuming it's all
%  synchrotron.  in reality it's not the case, nor is it a perfect
%  broken powerlaw, but this gives you a taste of how it can play in

% so let's refit by setting the first powerlaw spectrum to be a bit
%  inverted
%v^-alpha
%phoindex=1+alpha
%alpha=-.2
%phoindex=1-.2=.8

%  and the break at the extrapolation point from
%  the xrays (~1.e-3, will need to lower the limit) and see where that gets us
%  probably best is to go back to absorbed PL with cutoff not BB or
%  Gaussian, and get baseline fit

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*highecut(1)*bknpower(1)");
set_par(2,.6,1);
set_par(5,.4);
set_par(6,.82);
set_par(7,2.5e-4,0,1.e-5,1.e5);
set_par(8,1.6);
ec;
renorm_counts;

%got to chi2=18

%now put back other components

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*highecut(1)*(bknpower(1)+diskbb(1)+gaussian(1))");
renorm_counts;
fit_counts;

% did a bit of subplex, then back to lmdif
% get chi2 ~2.6, plot it:

plot_unfold({rid,pid,hid},popt);

% doesn't look bad, look at xrays

popt.xrange={2,250};
popt.yrange={2.e-3,.2,-5,5};
plot_unfold({pid,hid},popt);

% mismatch because slope is off, and constants trying to compensate
%  but can't.  When you see the need for more flux starting just at
%  around 10 keV, and a dip in residuals right before 10 keV, that
%  is often a signature of reflection

%add reflection component:

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*reflect(1,highecut(1)*(bknpower(1)+diskbb(1)+gaussian(1)))");

lp;

% give reflection fraction a modest value

set_par(14,.05);
ec;
    
% at this point could do bknpl etc but now let's talk about what you
%  would do to fit real physical models               

% What if you want to include a table model?  Here is an example of
%  some scripts that we wrote to include a stellar model to include in
%  fits.  Can be important because it's a big constraint on the
%  nonthermal fit we're interested in

% star_model basically just creates a function "star" that is the
% interpolated table read in from 
() = evalfile("star_model.sl",current_namespace);

% These are some other functions for setting up the model grid and
% calculating model along it

() = evalfile("eval_fun2_kev.sl");
() = evalfile("log_grid.sl");

% this variable will be used for plotting the component

(lo,hi) = log_grid(1.e-6,1.e-2,100);
variable starcomp=eval_fun2_keV("star",lo,hi,get_par("star(1).norm"));

% now let's add this component to the model.  It's just an additive component

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*reflect(1,highecut(1)*(bknpower(1)+diskbb(1)+gaussian(1)+ star(1)))");


linestyle(3);                       %dot-dashed
ohplot(lo,hi,starcomp*(lo+hi)/2./(hi-lo),18);  
linestyle(1);   % back to solid

%  doesn't really add much bc fit is ok, but in a source where star
%  really necessary, you'll see it

% Ok, now coming back from having slirped simplejet.f, use the model

() = evalfile("simplejet.sl");
fit_fun("simplejet");
lp;

plot_unfold({rid,pid,hid},popt);

%You can overplot the total jet contribution if you want, easiest to
% just use another plotting script at this point

.load plotscript

%  now before further fitting, best to cache the model over the entire
%   grid range defined in the data loading process

loval = _A(10^[-9:3:0.001]);
hival = make_hi_grid (loval);
cache_fun("simplejet",loval,hival);

% that makes a new model "simplejet_cache" you need to replace in your
%  fit function

fit_fun("simplejet_cache");

% eventually work way up to best fit using:

fit_fun("constant(Isis_Active_Dataset)*phabs(1)*highecut(1)*simplejet_cache(1)");


fit_fun("constant(Isis_Active_Dataset)*phabs(1)*reflect(1,highecut(1)*(simplejet_cache(1)+diskbb(1)+gaussian(1)))");
